﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace MUD.Client.Corba
{
    /// <summary>
    /// Client implementation for the corba sides
    /// </summary>
    public class ClientImpl :MarshalByRefObject, mud.common.idl.clients.Client
    {

        /// <summary>
        /// Pointer to the function to call to set a chat message
        /// </summary>
        private System.Threading.ParameterizedThreadStart chat;

        public const int SERVER_EVENT_CHAT = 0;
        public const int SERVER_EVENT_ROOM_INIT = 1;
        public const int SERVER_EVENT_MOVE = 2;
        public const int SERVER_EVENT_USE = 3;
        public const int SERVER_EVENT_DROP = 4;
        public const int SERVER_EVENT_REMOVE_OBJECT = 5;
        public const int SERVER_EVENT_ADD_OBJECT = 6;
        public const int SERVER_EVENT_ATTACK = 7;
        public const int SERVER_EVENT_PICKUP = 8;

        /// <summary>
        /// Events holder
        /// </summary>
        private WaitQueue events;

        /// <summary>
        /// Get the current wait event instance
        /// </summary>
        public WaitQueue MyEvents { 
            get { return events; }
            set { events = value; }
        }

        /// <summary>
        /// .NET remoting lifetime override. So the garbage collector doesn't discard the object
        /// </summary>
        /// <returns>null - Live forever</returns>
        public override object InitializeLifetimeService()
        {
            // live forever
            return null;
        }

        /// <summary>
        /// Create a new Client Implementation 
        /// </summary>
        public ClientImpl()
        {
            events = new WaitQueue(10);
        }

        #region Client Members

        public void pushEvent(mud.common.idl.clients._Event _event)
        {
            switch (_event.Discriminator)
            {
                case SERVER_EVENT_CHAT:
                    chat.Invoke(_event.GetchatEvent());
                    break;
                default:
                    events.Push(_event);
                    break;
            }
        }

        #endregion

        /// <summary>
        /// Set the internal message function
        /// </summary>
        /// <param name="chatstart">Pointer to the chatset function</param>
        internal void setChat(System.Threading.ParameterizedThreadStart chatstart)
        {
            this.chat = chatstart;
        }

        internal void setEventManager(System.Threading.ParameterizedThreadStart eventstart)
        {
            this.events.setReceiver(eventstart);
        }
    }
}
