﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using omg.org.CosNaming;
using mud.common.idl.clients;
using Ch.Elca.Iiop;
using Ch.Elca.Iiop.Services;
using System.Runtime.Remoting.Channels;
using mud.common.idl.servers;
using System.Runtime.Remoting;
using omg.org.CORBA;
using System.Windows.Forms;

namespace MUD.Client.Corba
{
    /// <summary>
    /// Handles the connection between the client and server.
    /// </summary>
    public partial class ServerConnection
    {
        /// <summary>
        /// Synchronization object
        /// </summary>
        private static Object _lock=new Object();
        /// <summary>
        /// Single instance of this class per client.
        /// </summary>
        private static ServerConnection _instance = null;

        /// <summary>
        /// Flag to indicate wether the orb is already running, so not to allow dual calls
        /// </summary>
        private Boolean running = false;

        /// <summary>
        /// Host of the orb
        /// </summary>
        private string nameServiceHost;

        /// <summary>
        /// Port of the orb
        /// </summary>
        private int nameServicePort;

        /// <summary>
        /// Server remote proxy 
        /// </summary>
        private Server server;

        /// <summary>
        /// Name for the local binding for the client
        /// </summary>
        private String name;

        /// <summary>
        /// The local implementation of the client.
        /// </summary>
        private ClientImpl client;

        /// <summary>
        /// Creates a new ServerConnection and initializes the orb using the given parameters
        /// </summary>
        /// <param name="args">Parameters to initialize the orb with</param>
        private ServerConnection(String[] args)
        {
            string nameServiceHost = "localhost";
            int nameServicePort = 10220;
            parseArgs(ref nameServiceHost, ref nameServicePort, args);

            this.nameServiceHost = nameServiceHost;
            this.nameServicePort = nameServicePort;

            name = "Client" + new Random().Next(10000);
           
        }

        /// <summary>
        /// Start the orb and make a connection both ways
        /// </summary>
        public void StartOrb()
        {
            lock (this)
            {
                if (running) return;

                running = true;
            }

            // register the channel

            IiopChannel callb = new IiopChannel(nameServicePort+1);
            ChannelServices.RegisterChannel(callb);

            // access COS nameing service
            CorbaInit init = CorbaInit.GetInit();

            NamingContext nameService = init.GetNameService(nameServiceHost, nameServicePort);
            NameComponent[] name = new NameComponent[] { new NameComponent("GameServer", "") };
            // get the reference to the Server
            this.server = (Server)nameService.resolve(name);

            client = new ClientImpl();
        }

        /// <summary>
        /// Get the current client implementation
        /// </summary>
        /// <returns>Client implementation</returns>
        public ClientImpl GetClient()
        {
            return client;
        }

        /// <summary>
        /// Return an instance of the class if it has been initialized already or throw an exception
        /// </summary>
        /// <returns>Instance of class</returns>
        public static ServerConnection getInstance()
        {
            lock (_lock)
            {
                if (_instance == null)
                    throw new Exception("Server connection has not yet been initialized.");

                return _instance;
            }
        }

        /// <summary>
        /// Return an instance of the class if already there or create a new one
        /// </summary>
        /// <param name="args">Parmeters to initialize the system with</param>
        /// <returns>Instance of class</returns>
        public static ServerConnection getInstance(String[] args)
        {
            lock (_lock)
            {
                if (_instance == null)
                {
                    _instance = new ServerConnection(args);
                    _instance.StartOrb();
                }

                return _instance;
            }
        }

        /// <summary>
        /// Override the local variables with the given commandline args
        /// </summary>
        /// <param name="host">address of the computer with the orb on it</param>
        /// <param name="port">portnumber on where the orb is running</param>
        /// <param name="args">passes commandline arguments</param>
        private void parseArgs(ref string host, ref int port, string[] args)
        {
            if (args == null) return;

            try
            {
                for (int i = 0; i < args.Length; i++)
                {
                    switch (args[i].ToLower())
                    {
                        case "--orbinitialport":
                            port = Int32.Parse(args[++i]);
                            break;
                        case "--orbinitialhost":
                            host = args[++i];
                            break;
                        default:
                            throw new ArgumentException("invalid argument");
                    }
                }
            }
            catch (Exception)
            {
                Console.WriteLine("Unrecognized commandline option. \r\n Options: \r\n\t --ORBInitialPort <port> \r\n\t --ORBInitialHost <host>");
                throw new ArgumentException("invalid argument");
            }
        }

        /// <summary>
        /// Set the internal message function
        /// </summary>
        /// <param name="chatstart">Pointer to the chatset function</param>
        internal void setChat(System.Threading.ParameterizedThreadStart chatstart)
        {
            this.client.setChat(chatstart);
        }


        internal void setEventManager(System.Threading.ParameterizedThreadStart eventstart)
        {
            this.client.setEventManager(eventstart);
        }

        internal int getSessionId()
        {
            return security_token;
        }
    }
}
