﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using MUD.Client.Corba;

namespace MUD.Client
{
    public partial class LoginForm : Form
    {
        double duration = 100;
        int steps = 30;
        int step = 0;
        int spin;

        System.Timers.Timer error_timer;
        ServerConnection connection;

        public delegate void UpdateUIDelegate();
        public UpdateUIDelegate error;

        public LoginForm()
        {
            //Control.CheckForIllegalCrossThreadCalls=false;

            error = new UpdateUIDelegate(updateGUI);

            InitializeComponent();
            password.ReadyEventChange +=new PreviewTextbox.ReadyEvent(ReadyEventChange);
            username.ReadyEventChange += new PreviewTextbox.ReadyEvent(ReadyEventChange);
            error_timer = new System.Timers.Timer(duration / steps);
            error_timer.Elapsed += new System.Timers.ElapsedEventHandler(error_timer_Elapsed);
            
            this.VisibleChanged += new EventHandler(LoginForm_VisibleChanged);

            connection = ServerConnection.getInstance();
        }

        void LoginForm_VisibleChanged(object sender, EventArgs e)
        {
            //workaround for a bug in my custom button code. cannot set this at design time or the animation won't update
            btnLogin.EnabledExtended = false;
        }

        void error_timer_Elapsed(object sender, System.Timers.ElapsedEventArgs e)
        {
            this.Invoke(error);
        }

        void updateGUI()
        {
            step++;
            float max = steps / 2.0f;
            if (step <= max)
            {
                //down animation
                pnlError.Top = (int)(-pnlError.Height * (1 - (step / max)));
            }
            else
            {
                //up animation
                spin--;
                if (spin > 0)
                {
                    step--;
                }
                else
                {
                    pnlError.Top = (int)(-pnlError.Height * ((step - max) / max));
                }
            }

            if (step == steps)
            {
                error_timer.Stop();
                pnlError.Visible = false;
                pnlBottomColor.BackColor = panel1.BackColor;
            }
        }

        void ReadyEventChange()
        {
            btnLogin.EnabledExtended=password.isReady && username.isReady;
        }

        private void btnCancel_Click(object sender, EventArgs e)
        {
            Application.Exit(); 
        }

        private void showError(string error)
        {
            pnlError.Top = -pnlError.Height;
            pnlError.Visible = true;
            lblError.Text = error;
            pnlBottomColor.BackColor = pnlError.BackColor;
            error_timer.Start();
            step = 0;
            spin = 100;
        }

        private void btnLogin_Click(object sender, EventArgs e)
        {
            try
            {
                bool loggedin = connection.Login(username.Text, password.Text);
                if (!loggedin)
                    showError("Login failed. Please check the supplied username \n and password");
                else
                {
                    this.Close();
                }
            }
            catch (Exception ex)
            {
                showError("Error: " +ex.Message);
            }
        }
    }
}
