﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using MUD.Client.Corba;
using mud.common.idl.clients.Event_package;
using mud.common.idl.clients;
using System.Xml;

namespace MUD.Client
{
    public partial class MainWindow : Form
    {
        ServerConnection server;
        String buff = "<div>Welcome to the multi useless dungeon.</div>";

        public delegate void UpdateChatUI(ChatEvent message);
        public UpdateChatUI chat;

        public delegate void updateEventUIs(_Event evnt);
        public updateEventUIs events;

        AI player;

        public MainWindow()
        {
            InitializeComponent();
            chat = new UpdateChatUI(newChatEvent);
            events = new updateEventUIs(newEvents);
            this.VisibleChanged += new EventHandler(MainWindow_VisibleChanged);
            this.FormClosing += new FormClosingEventHandler(MainWindow_FormClosing);

            chatWindow.Navigate("about:blank");
            chatWindow.DocumentText=buff;
            message.ReadyEventChange += new PreviewTextbox.ReadyEvent(message_ReadyEventChange);

            server = ServerConnection.getInstance();
            player = new AI(800,server);
            player.setId(server.getSessionId());
            player.StartAI(true);
        }

        void MainWindow_FormClosing(object sender, FormClosingEventArgs e)
        {
            server.GetClient().MyEvents.Dispose();
        }   

        void message_ReadyEventChange()
        {
            btnSend.EnabledExtended = message.isReady;
        }

        void MainWindow_VisibleChanged(object sender, EventArgs e)
        {
            btnSend.EnabledExtended = false;
        }

        public void newChatEvent(ChatEvent message)
        {
            String msg = new String(message.message).Replace("\0", "");
            this.buff = "<div>" + msg + "</div>" + this.buff;
            chatWindow.DocumentText=this.buff;

            Application.DoEvents();
        }

        public void newEvents(_Event evnt)
        {
            TextBox current = null;
            String xml = "";
            switch (evnt.Discriminator)
            {
                case ClientImpl.SERVER_EVENT_ADD_OBJECT:
                    current = txtAddObject;
                    xml = evnt.GetaddObjectEvent().object_xml;
                    player.add(evnt.GetaddObjectEvent().object_xml);
                    break;
                case ClientImpl.SERVER_EVENT_ATTACK:
                    current = txtAttack;
                    AttackEvent v = evnt.GetattackEvent();
                    xml = "Attacker " + v.attacker_id + " :" + v.attacker_damage + "\r\n";
                    xml += "Opponent " + v.defender_id + " :" + v.defender_damage;
                    break;
                case ClientImpl.SERVER_EVENT_DROP:
                    current = txtDrop;
                    xml = "Dropped: " + evnt.GetdropEvent().item_id + " By: " + evnt.GetdropEvent().creature_id;
                    break;
                case ClientImpl.SERVER_EVENT_MOVE:
                    current = txtMove;
                    MoveEvent x = evnt.GetmoveEvent();
                    xml = "Move " + x.object_id + " in Direction: " + x.direction + " , (x,y)=" + x.x + "," + x.y;
                    player.move(x.object_id, x.x, x.y, x.direction);
                    break;
                case ClientImpl.SERVER_EVENT_PICKUP:
                    current = txtPickup;
                    xml = evnt.GetpickupEvent().item_xml;
                    break;
                case ClientImpl.SERVER_EVENT_REMOVE_OBJECT:
                    current = txtRemove;
                    xml = "Item Removed: " + evnt.GetremoveObjectEvent().object_id;
                    player.remove(evnt.GetremoveObjectEvent().object_id);
                    break;
                case ClientImpl.SERVER_EVENT_ROOM_INIT:
                    current = txtRoominit;
                    xml = evnt.GetroomInitEvent().room_xml;
                    player.init(xml,false);
                    break;
                case ClientImpl.SERVER_EVENT_USE:
                    current = txtUse;
                    xml = "Player " + evnt.GetuseEvent().creature_id + " used " + evnt.GetuseEvent().item_id;
                    break;
                default:
                    break;
            }

            current.Text = xml + "\r\n" + current.Text;

            Application.DoEvents();
        }

        public void newMessage(Object message)
        {
            try
            {
                this.Invoke(chat,new object[] { (ChatEvent)message});
            }
            catch (Exception)
            {
                throw;
            }
        }

        public void newEvent(Object evnt){
            try
            {
                this.Invoke(events, new object[] { (_Event)evnt });
            }
            catch (Exception)
            {
                throw;
            }
        }

        private void MainWindow_Load(object sender, EventArgs e)
        {

        }

        private void btnSend_Click(object sender, EventArgs e)
        {
            server.SendMessage(message.Text);
            message.Text = "";
        }

        private void btnUsers_Click(object sender, EventArgs e)
        {
            String[] users = server.GetUsers();
            String result = "";

            for (int i = 0; i < users.Length; i++)
            {
                result += users[i];
                if (i < users.Length - 1)
                    result += ", ";
            }

            newChatEvent(new ChatEvent(("<strong>Users list</strong>: <i>" + result + "</i>").ToCharArray()));
        }
    }
}
