using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace MUD.Client
{
    public partial class PreviewTextbox : XP_themed_style.XpTextBox
    {
        bool ready = false;
        bool passworded = false;
        char passChar;

        public bool isReady
        {
            get { return ready; }
        }

        public delegate void ReadyEvent();
        public event ReadyEvent ReadyEventChange;

        public PreviewTextbox()
        {
            InitializeComponent();
            this.Enter += new EventHandler(PreviewTextbox_Enter);
            this.Leave += new EventHandler(PreviewTextbox_Leave);
            oldfont = this.Font;
            smallfont = new Font(this.Font.FontFamily, 7, FontStyle.Regular);
            Passinit();
            base.TextChanged += new TextChangedEventHandler(PreviewTextbox_TextChanged);
        }

        void PreviewTextbox_TextChanged(object sender, EventArgs e)
        {
            OnTextChanged(e);
        }

        void Passinit()
        {
            passworded = !PasswordChars.Equals('\0');
            passChar = PasswordChars;
            if (passworded)
                PasswordChars = '\0';
        }

        void PreviewTextbox_Leave(object sender, EventArgs e)
        {
            if (this.Text.Trim().Length == 0)
            {
                ready = false;
                if (passworded)
                    PasswordChars = '\0';
                this.Text = prevtxt;
                this.Font = smallfont;
                this.ForeColor = forColor;
            }
            else
                ready = true;

            sendReady();
        }

        private void sendReady()
        {
            if (ReadyEventChange != null)
                ReadyEventChange.Invoke();
        }

        protected override void OnTextChanged(EventArgs e)
        {
            ready = (this.Text.Trim().Length > 0) && !Text.Equals(prevtxt);
            base.OnTextChanged(e);
            sendReady();
        }

        public void reset()
        {
            Text = "";
            PreviewTextbox_Leave(this, new EventArgs());
            sendReady();
        }

        void PreviewTextbox_Enter(object sender, EventArgs e)
        {
            if (this.Text.Equals(prevtxt))
            {
                this.Text = "";
                this.Font = oldfont;
                this.ForeColor = oldforground;
                if (passworded)
                    PasswordChars = passChar;
            }
            sendReady();
        }

        string prevtxt = "";
        Color oldforground = Color.Black;
        Color forColor = Color.Gray;
        Font oldfont;
        Font smallfont;

        public string PreviewText
        {
            get{
                return prevtxt;
            }
            set
            {
                prevtxt = value;
                this.Text = value;
                this.Font = smallfont;
                this.ForeColor = forColor;
                Passinit();
            }
        }

        protected override void OnPaint(PaintEventArgs pe)
        {
            // TODO: Add custom paint code here

            // Calling the base class OnPaint
            base.OnPaint(pe);
        }
   
    }
}
