using System;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Drawing.Imaging;
using System.Drawing.Custom;
using System.Collections.Generic;
using System.Text;

namespace System.Drawing.Custom
{
    public static class Paths
    {
        public static GraphicsPath getRoundedPath(float X, float Y, float width, float height, float radius)
        {
            GraphicsPath gp = new GraphicsPath();
            gp.AddLine(X + radius, Y, X + width - (radius * 2), Y);
            gp.AddArc(X + width - (radius * 2), Y, radius * 2, radius * 2, 270, 90);
            gp.AddLine(X + width, Y + radius, X + width, Y + height - (radius * 2));
            gp.AddArc(X + width - (radius * 2), Y + height - (radius * 2), radius * 2, radius * 2, 0, 90);
            gp.AddLine(X + width - (radius * 2), Y + height, X + radius, Y + height);
            gp.AddArc(X, Y + height - (radius * 2), radius * 2, radius * 2, 90, 90);
            gp.AddLine(X, Y + height - (radius * 2), X, Y + radius);
            gp.AddArc(X, Y, radius * 2, radius * 2, 180, 90);
            gp.CloseFigure();
            return gp;
        }
    }
    public static class GFX
    {
        public static void drawRoundedRect(Graphics e,Pen p, float x, float y, float width, float height, float radius)
        {
            e.DrawPath(p,Paths.getRoundedPath(x, y, width, height, radius));
        }
        public static void fillRoundedRect(Graphics e, Brush b, float x, float y, float width, float height, float radius)
        {
            e.FillPath(b, Paths.getRoundedPath(x, y, width, height, radius));
        }
        public static void RoundedRect(Graphics e, Pen p,Brush b, float x, float y, float width, float height, float radius)
        {
            drawRoundedRect(e, p, x, y, width, height, radius);
            fillRoundedRect(e, b, x, y, width, height, radius);
        }
    }
    public static class Images
    {
        public static Image grayscale(Bitmap input)
        {
            Image img = input;
            Bitmap bm = new Bitmap(img.Width, img.Height);
            Graphics g = Graphics.FromImage(bm);

            ColorMatrix cm = new ColorMatrix(new Single[][]
               {new Single[] {0.3f, 0.3f, 0.3f, 0, 0}, 
                new Single[] {0.59f, 0.59f, 0.59f, 0, 0}, 
                new Single[] {0.11f, 0.11f, 0.11f, 0, 0}, 
                new Single[] {0, 0, 0, 1, 0}, 
                new Single[] {0, 0, 0, 0, 1}});
            ImageAttributes ia = new ImageAttributes();
            ia.SetColorMatrix(cm);
            ia.SetGamma(1.5f);
            g.DrawImage(img, new Rectangle(0, 0, img.Width, img.Height), 0, 0, img.Width, img.Height, GraphicsUnit.Pixel, ia);
            return bm;
        }
        public static Image blackAndWhite(Bitmap input){
        Bitmap bm = (Bitmap)grayscale(input);
        Image img = bm;
        Graphics g = Graphics.FromImage(bm);
        ImageAttributes ia = new ImageAttributes();
        ia.SetGamma(20);
        ia.SetNoOp(ColorAdjustType.Brush);
        g.DrawImage(img, new Rectangle(0, 0, img.Width, img.Height), 0, 0, img.Width, img.Height, GraphicsUnit.Pixel, ia);
        bm.MakeTransparent(Color.White);
        BitmapData bit = bm.LockBits(new Rectangle(0, 0, bm.Width, bm.Height), ImageLockMode.ReadWrite, PixelFormat.Format32bppArgb);
        bit.PixelFormat = PixelFormat.Max;
        bm.UnlockBits(bit);
        return bm;
        }
    }

}
