package mud.client;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.SwingConstants;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

import mud.client.corba.ServerConnection;

/**
 * A simple swing component to show received chat messages from the server an other players 
 */

@SuppressWarnings("serial")
public class ChatView extends JPanel {

	private HtmlLabel chatwindow;
	private JButton send, user;
	private JTextField input;
	private ServerConnection server;
	private JFrame frame;
	private JList users = null;
	private JScrollPane listScroller = null;
	private boolean showUsers_b;

	public ChatView(ServerConnection s, JFrame frame) {
		this.frame = frame;

		chatwindow = new HtmlLabel();
		chatwindow.setVerticalAlignment(SwingConstants.TOP);
		JScrollPane chatPane = new JScrollPane(chatwindow, JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
				JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
		chatPane.setPreferredSize(new Dimension(1, 85));

		send = new JButton("Send");
		user = new JButton("Users");
		input = new JTextField();

		send.setEnabled(false);
		input.getDocument().addDocumentListener(new DocumentListener() {
			public void removeUpdate(DocumentEvent arg0) {
				updateEnabledState();
			}
			public void insertUpdate(DocumentEvent arg0) {
				updateEnabledState();
			}
			public void changedUpdate(DocumentEvent arg0) {
				updateEnabledState();
			}
			public void updateEnabledState() {
				send.setEnabled(input.getText().trim().length() > 0);
			}
		});

		this.server = s;

		ActionListener sl = new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				String msg = input.getText().trim();
				if (msg.length() > 0) {
					try {
						server.sendMessage(msg);
						input.setText("");
					} catch (Exception e) {
						e.printStackTrace();
					}
				}
			}
		};

		send.addActionListener(sl);
		input.addActionListener(sl);

		user.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (showUsers_b)
					hideUsers();
				else
					showUsers();

				showUsers_b = !showUsers_b;
			}
		});

		send.setPreferredSize(new Dimension(100, 20));
		user.setPreferredSize(new Dimension(100, 20));
		JPanel pbuttons = new JPanel();
		pbuttons.add(send);
		pbuttons.add(user);

		JPanel panels = new JPanel();
		panels.setLayout(new BorderLayout());
		panels.add(input, BorderLayout.CENTER);
		panels.add(pbuttons, BorderLayout.EAST);

		setLayout(new BorderLayout());
		add(chatPane, BorderLayout.CENTER);
		add(panels, BorderLayout.SOUTH);

		chatwindow.appendText("Welcome to the multi useless dungeon..");
	}

	public void appendText(String message) {
		chatwindow.appendText(message);
	}

	/** Show the list of users currently logged in */
	public void showUsers() {
		try {
			if (users != null) {
				return;
			}
			users = new JList(server.getUsers());
			users.setSelectionMode(ListSelectionModel.SINGLE_INTERVAL_SELECTION);
			users.setLayoutOrientation(JList.VERTICAL);
			users.setVisibleRowCount(-1);

			listScroller = new JScrollPane(users);
			listScroller.setPreferredSize(new Dimension(150, 1));
			listScroller.setVisible(true);
			frame.add(listScroller, BorderLayout.EAST);
			frame.pack();
		} catch (Exception ex) {
		}
	}

	public void hideUsers() {
		if (users == null) {
			return;
		}
		users.setVisible(false);
		frame.remove(listScroller);
		users = null;
		listScroller = null;
		frame.pack();
	}
}
