package mud.client;

import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.Font;
import java.awt.FontFormatException;
import java.awt.GradientPaint;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.io.File;
import java.io.IOException;

public class GuiUtil {
	
	public static final Color SELECTION_1 = new Color(129, 170, 231);
	public static final Color SELECTION_2 = new Color( 56, 111, 199);
	public static final Color GRADIENT_1  = SELECTION_1;
	public static final Color GRADIENT_2  = new Color(104, 138, 186);
	public static final Color GRADIENT_3  = new Color( 61,  93, 137);
	
	public static final Font loadFont(String name) {
		try {
			return Font.createFont(Font.TRUETYPE_FONT, new File(name));
		} catch (FontFormatException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		return new Font("arial", Font.PLAIN, 12);
	}

	public static void recursiveSetEnabled(Component c, boolean e) {		
		if (c instanceof Container) {
			for (Component child : ((Container)c).getComponents()) {
				child.setEnabled(e);
				
				recursiveSetEnabled(child, e);
			}
		}
	}
	public static void recursiveDisableRepaint(Component c) {
		c.setIgnoreRepaint(true);
		if (c instanceof Container) {
			for (Component child : ((Container)c).getComponents()) {
				recursiveDisableRepaint(child);
			}
		}
	}

	public static Color setAlpha(Color c, int alpha) {
		return new Color((c.getRGB() & 0x00FFFFFF) | (alpha << 24), true);
	}

	public static void drawSelectionRect(Graphics g, int x, int y, int w, int h) {
		drawSelectionRect(g, x, y, w, h, SELECTION_1, SELECTION_2);
	}
	public static void drawSelectionRect(Graphics g, int x, int y, int w, int h, Color c1, Color c2) {
		if (c1 != null) {
			g.setColor(c1);
			g.fillRect(x + 2, y + 2, w - 4, h - 4);
		}
		
		if (c2 != null) {
			g.setColor(c2);
			g.drawRect(x + 1, y + 1, w - 3, h - 3);
			g.drawLine(x + 1, y, x + w - 2, y);
			g.drawLine(x + 1, y + h - 1, x + w - 2, y + h - 1);
			g.drawLine(x, y + 1, x, y + h - 2);
			g.drawLine(x + w - 1, y + 1, x + w - 1, y + h - 2);
	
			g.setColor(setAlpha(c2, 100));
			g.drawLine(x, y, x + 1, y + 1);
			g.drawLine(x + w - 1, y, x + w - 2, y + 1);
			g.drawLine(x, y + h - 1, x + 1, y + h - 2);
			g.drawLine(x + w - 1, y + h - 1, x + w - 2, y + h - 2);
		}
	}
	public static void drawGradientSelectionRect(Graphics graphics, int x, int y, int w, int h) {
		Graphics2D g = (Graphics2D)graphics;

		g.setColor(GRADIENT_3);
		g.drawRect(x + 1, y + 1, w - 3, h - 3);
		g.drawLine(x + 1, y, x + w - 2, y);
		g.drawLine(x + 1, y + h - 1, x + w - 2, y + h - 1);
		g.drawLine(x, y + 1, x, y + h - 2);
		g.drawLine(x + w - 1, y + 1, x + w - 1, y + h - 2);

		GradientPaint gradient = new GradientPaint(
				2, 2,     GRADIENT_1,
				2, 2 + h, GRADIENT_2);
		g.setPaint(gradient);
		g.fillRect(2, 2, w - 4, h - 4);	
	}

}
