package mud.client;

import java.awt.event.KeyEvent;
import java.util.Collection;
import java.util.LinkedList;
import java.util.List;

import javax.swing.SwingUtilities;

import mud.client.messages.AttackRequest;
import mud.client.messages.Message;
import mud.client.messages.MoveRequest;
import mud.client.view.GameView;
import mud.client.view.contextmenu.ContextMenu;
import mud.client.view.contextmenu.ContextMenuEvent;
import mud.common.items.Item;
import mud.common.objects.InventoryListener;
import mud.common.objects.Player;

/** Contains all the code required to control the user's own player */
public class LocalPlayer {
	
	private GameView gameView;
	private Player player;
	private InventoryListener inventoryListener;
	private ContextMenu contextMenu;
	private boolean menuOpen;
	
	private List<ContextMenuEvent> contextEvents;
	
	public LocalPlayer(GameView gameView) {
		this.gameView = gameView;
		
		contextMenu = new ContextMenu();
		contextEvents = new LinkedList<ContextMenuEvent>();
		
		inventoryListener = new InventoryListener() {
			public void onItemAdded(Item i) {
			}
			public void onItemRemoved(Item i) {
				//Dismiss the contextmenu when the inventory is changed. Otherwise the inventory displayed
				//is no longer consistent with the actual inventory.
				SwingUtilities.invokeLater(new Runnable() {
					public void run() {
						contextMenu.setVisible(false);
					}
				});
			}
		};
	}
	
	//Functions
	public void update(UserInput input, Collection<Message> sendQueue) {
		if (player == null || player.getActivity() != null) {
			return;
		}
		
		int x = player.getX();
		int y = player.getY();

		menuOpen = contextMenu.isVisible();
		if (input.consumeKey(KeyEvent.VK_Z)) {
			if (!menuOpen) {
				sendQueue.add(new AttackRequest());
			}
		} else if (input.consumeKey(KeyEvent.VK_X)) {
			input.clear();

			SwingUtilities.invokeLater(new Runnable() {
				public void run() {
					if (!contextMenu.isVisible()) {
						contextMenu.show(LocalPlayer.this, gameView, gameView.getWidth()/2, gameView.getHeight()/2);
					} else {
						contextMenu.setVisible(false);
					}
				}
			});
		}
						
		if (input.isKeyHeld(KeyEvent.VK_UP)) {
			sendQueue.add(new MoveRequest(x, y-1));
		} else if (input.isKeyHeld(KeyEvent.VK_DOWN)) {
			sendQueue.add(new MoveRequest(x, y+1));
		} else if (input.isKeyHeld(KeyEvent.VK_LEFT)) {
			sendQueue.add(new MoveRequest(x-1, y));
		} else if (input.isKeyHeld(KeyEvent.VK_RIGHT)) {
			sendQueue.add(new MoveRequest(x+1, y));			
		}
		
		synchronized (contextEvents) {
			while (!contextEvents.isEmpty()) {
				try {
					sendQueue.add(contextEvents.remove(0).getMessage());
				} catch (RuntimeException re) {
					re.printStackTrace(); //For if the event is no longer applicable
				}				
			}
		}
	}
	
	public void addContextMenuEvent(ContextMenuEvent event) {
		synchronized (contextEvents) {
			contextEvents.add(event);
		}
	}
	
	//Getters
	public Player getPlayer() {
		return player;
	}
	
	//Setters
	public void setPlayer(Player p) {
		if (player != p) {
			if (player != null) player.getInventory().removeListener(inventoryListener);
			
			player = p;
			
			if (player != null) player.getInventory().addListener(inventoryListener);
		}
	}
	
}
