package mud.client;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.GridLayout;
import java.awt.LayoutManager;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.List;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.border.EmptyBorder;

@SuppressWarnings("serial")
public class MessageBox {	
		
	protected int selectedOption;
	protected List<ActionListener> listeners;
	
	private String title;
	private int optionButtonWidth;
	
	private JPanel panel;
	private OptionPanel optionPanel;
	
	public MessageBox() {
		this(null);
	}
	public MessageBox(String title) {
		this(title, null);
	}
	public MessageBox(String title, Object message) {
		if (title != null) {
			this.title = title;
		}
		
		this.selectedOption = -1;
		this.listeners = new ArrayList<ActionListener>();		
		this.optionButtonWidth = 400;
		
		panel = new JPanel();
		panel.setLayout(new BorderLayout());
		
		if (message instanceof JComponent) {
			panel.add((JComponent)message);
		} else if (message != null) {
			JLabel label = new JLabel("<html><div style='width: " + getButtonWidth() + "'>"+message.toString()+"</div></html>");
						
			JPanel labelPanel = new JPanel(new FlowLayout(FlowLayout.LEFT));
			labelPanel.setBackground(new Color(240, 240, 240));
			labelPanel.add(label);
			
			JPanel labelPanel2 = new JPanel(new BorderLayout());
			labelPanel2.setBorder(new EmptyBorder(10, 10, 5, 10));
			labelPanel2.add(labelPanel, BorderLayout.CENTER);
			
			panel.add(labelPanel2, BorderLayout.CENTER);
		}
		
		createOptionPanel();
	}
	
	public void fireButtonPress(int index) {
		buttonPressed(index);
	}
	
	public int showMessage() {
		return showMessage(null);
	}
	public int showMessage(Component component) {
		final JDialog dialog;
		if (component != null) {
			dialog = new JDialog(JOptionPane.getFrameForComponent(component));
		} else {
			dialog = new JDialog();
		}
		
		addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				dialog.setVisible(false);
			}
		});

		optionPanel.setButtonsCentered(optionPanel.getNonButtonOptionCount() == 0);
		
		dialog.add(panel);		
		dialog.pack();		
		dialog.setLocationRelativeTo(component);
		
		dialog.setTitle(title);
		dialog.setResizable(false);
		dialog.setModal(true);			
		dialog.setVisible(true);
		dialog.dispose();
		
		return selectedOption;
	}
	protected void buttonPressed(int index) {
		selectedOption = index;
		
		notifyListeners(index);		
	}
	protected void createOptionPanel() {
		optionPanel = new OptionPanel(true);		
		panel.add(optionPanel, BorderLayout.SOUTH);
	}
	
	public void addOption(String option, String description) {
		optionPanel.addOption(option, description);
	}
	public void addButton(String option, String description) {
		optionPanel.addButton(option, description);
	}
	
	//Listener Functions
	public void addActionListener(ActionListener al) {
		listeners.add(al);
	}
	public void removeActionListener(ActionListener al) {
		listeners.remove(al);
	}
	public void notifyListeners(int selectedIndex) {
		ActionEvent event = new ActionEvent(this, selectedIndex, "buttonPressed");
		
		ActionListener l[] = new ActionListener[listeners.size()];
		listeners.toArray(l);
		for (ActionListener al : l) {
			al.actionPerformed(event);
		}
	}
	
	//Getters
	public int getButtonWidth() {
		return optionButtonWidth;
	}
	public String getTitle() {
		return title;
	}

	//Setters
	public void setButtonWidth(int buttonWidth) {
		this.optionButtonWidth = buttonWidth;
	}
	public void setTitle(String title) {
		this.title = title;
	}
	public void removeAllOptions() {
		if (optionPanel != null) {
			panel.remove(optionPanel);
		}
		
		createOptionPanel();
	}
	
	//Inner Classes
	private class OptionPanel extends JPanel {
		
		private List<String> options;
		private List<String> descriptions;

		private JPanel optionPanel;
		private JPanel buttonPanel;
		private LayoutManager layout;

		public OptionPanel(boolean makeButtonsEqualSize) {
			this.options = new ArrayList<String>();
			this.descriptions = new ArrayList<String>();
			
			optionPanel = new JPanel();
			optionPanel.setMaximumSize(new Dimension(600, 9999));
			optionPanel.setBorder(new EmptyBorder(10, 10, 10, 10));
			optionPanel.setVisible(false);
			if (makeButtonsEqualSize) {
				layout = new GridLayout(-1, 1, 5, 5);
			} else {
				layout = new BoxLayout(optionPanel, BoxLayout.Y_AXIS);
			}
			optionPanel.setLayout(layout);
			
			buttonPanel = new JPanel(new FlowLayout(FlowLayout.RIGHT));
			buttonPanel.setVisible(false);
			
			setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
			add(optionPanel);
			add(buttonPanel);
		}
		
		//Functions		
		public void addButton(String option, String description) {
			addOption2(option, description, true);
		}
		public void addOption(String option, String description) {
			addOption2(option, description, false);
		}
		public void addOption2(String option, String description, boolean asButton) {
			final int index = options.size();

			if (option == null) {
				option = "";
			}
			if (description == null) {
				description = "";
			}
			
			options.add(option);
			descriptions.add(description);
			
			JButton button;
			if (asButton) {
				button = new JButton(option);
				button.setToolTipText(description);
			} else {
				button = new JButton("<html><b>" + option + "</b><br><br><div style='width: "
					+ getButtonWidth() + "'>" + description + "</p></html>") {
					
					public void paint(Graphics graphics) {
						Graphics2D g = (Graphics2D)graphics;						
						int w = getWidth();
						int h = getHeight();
						
						g.setBackground(getParent().getBackground());
						g.clearRect(0, 0, w, h);
						if (!model.isRollover()) {
							Color c1 = new Color(240, 240, 240, 127);
							Color c2 = new Color(127, 127, 127, 127);
							GuiUtil.drawSelectionRect(g, 0, 0, w, h, c1, c2);
						} else {
							GuiUtil.drawGradientSelectionRect(g, 0, 0, w, h);
						}
						
						if (model.isArmed()) {
							g.translate(1, 1);
						}
						
						super.paint(g);
						
						if (model.isArmed()) {
							g.translate(-1, -1);
						}
					}
				};

				button.setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
				button.setBorder(new EmptyBorder(5, 5, 5, 5));
				button.setOpaque(false);
				button.setBackground(new Color(0, 0, 0, 0));
			}
			
			button.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent event) {
					buttonPressed(index);
				}
			});

			if (asButton) {
				buttonPanel.setVisible(true);
				buttonPanel.add(button);
			} else {
				optionPanel.setVisible(true);

				if (index > 0 && layout instanceof BoxLayout) {
					optionPanel.add(Box.createVerticalStrut(5));
				}
				
				button.setMinimumSize(new Dimension(getButtonWidth(), 1));
				optionPanel.add(button);
			}
		}
		
		//Getters
		public int getNonButtonOptionCount() {
			return optionPanel.getComponentCount();
		}
		
		//Setters
		public void setButtonsCentered(boolean centered) {
			FlowLayout layout = (FlowLayout)buttonPanel.getLayout();
			layout.setAlignment(centered ? FlowLayout.CENTER : FlowLayout.RIGHT);
			validate();
		}
	}

}
