package mud.client;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.DisplayMode;
import java.awt.GraphicsDevice;
import java.awt.GraphicsEnvironment;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.JFrame;

import mud.client.corba.ClientImpl;
import mud.client.corba.ServerConnection;
import mud.client.view.GameView;
import mud.common.Constants;

@SuppressWarnings("serial")
public class Program extends JFrame {

	private GameLoop loop;
	private GameView view;
	private ServerConnection server;
	private ChatView chatview;
	
	public Program(String[] args) {
		server = new ServerConnection(args);

		view = new GameView();
		loop = new GameLoop(view);

		chatview = new ChatView(server,this);
		chatview.setPreferredSize(new Dimension(1, 120));

		setLayout(new BorderLayout());
		add(view, BorderLayout.CENTER);
		add(chatview, BorderLayout.SOUTH);
		

		addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				stop();
				dispose();
				System.exit(0);
			}
		});
		
		
		setTitle(String.format("Multi Useless Dungeon %s", Constants.versionString));

		GraphicsEnvironment ge = GraphicsEnvironment.getLocalGraphicsEnvironment();
		GraphicsDevice gd = ge.getDefaultScreenDevice();
		boolean fullscreen = false;
		if (Constants.START_FULLSCREEN && gd.isFullScreenSupported()) {
			try {
				setUndecorated(true);

				if (gd.isDisplayChangeSupported()) {
					for (DisplayMode dm : gd.getDisplayModes()) {
						if (dm.getWidth() == 640 && dm.getHeight() == 480) {
							gd.setDisplayMode(dm);
							break;
						}
					}
				}

				gd.setFullScreenWindow(this);
				fullscreen = true;
			} catch (Exception e) {
				e.printStackTrace();
			}
		}

		if (!fullscreen) {
			setUndecorated(false);
			pack();
			setLocationRelativeTo(null);
		}
	}

	// Functions
	public static void main(String args[]) {
		Program p = new Program(args);
		p.start();
	}

	public void start() {
		server.start();
		
		ClientImpl ci = server.getClient();		
		loop.start(server);
		ci.init(loop, chatview);

		LoginView login = new LoginView(server);
		if (login.loginSucceeded()) {
			loop.setPlayerId(server.getSecurityToken());
			
			Runtime.getRuntime().addShutdownHook(new Thread(new Runnable() {
				public void run() {
					try {
						server.logout();
					} catch (Exception e) { }
				}
			}));

			setVisible(true);
		} else {
			stop();
		}
	}

	public void stop() {
		loop.stop();
		server.stop();
	}

	// Getters

	// Setters

}
