package mud.client;

import java.util.Collection;
import java.util.LinkedList;
import java.util.List;

import mud.client.corba.ServerConnection;
import mud.client.messages.Message;
import mud.common.exceptions.AuthenticationException;
import mud.common.idl.servers.ServerPackage.InvalidSecurityToken;

/** Sends messages to the server asynchronously */
public class SendLoop implements Runnable {

	private Thread thread;
	private boolean running;
	private boolean quit;
	
	private ServerConnection serverConnection;
	private List<Message> sendQueue;
	
	public SendLoop() {
	}
	
	//Functions
	public void start(ServerConnection sc) {
		if (running) throw new IllegalStateException("SendLoop.start() :: Already running");
		running = true;
		quit = false;
		
		serverConnection = sc;
		sendQueue = new LinkedList<Message>();
		thread = new Thread(this, "Client::SendLoop");		
		thread.start();
	}
	public void stop() {		
		quit = true;
	}

	/** Add a new batch of messages */
	public void update(Collection<Message> newMessages) {
		if (!newMessages.isEmpty()) {
			synchronized (this) {
				sendQueue.addAll(newMessages);
				notify();
			}
		}
	}
	
	public void run() {
		while (!quit) {
			synchronized (this) {
				while (!sendQueue.isEmpty()) {
					Message m = sendQueue.remove(0);
					try {
						m.execute(serverConnection);
					} catch (InvalidSecurityToken e) {
						e.printStackTrace();
					} catch (AuthenticationException e) {
						e.printStackTrace();
					}
				}
			
				try {
					wait();
				} catch (InterruptedException ie) { }
			}
		}

		sendQueue = null;
		serverConnection = null;
		running = false;
	}
	
	//Getters
	
	//Setters
	
}
