package mud.client;

import java.util.Collection;
import java.util.HashSet;
import java.util.Set;

/** Contains all the keys pressed/released this frame */
public class UserInput {
	
	private boolean enabled;
	private Set<Integer> pressed;
	private Set<Integer> consumed;
	
	public UserInput() {
		enabled = true;
		pressed = new HashSet<Integer>();
		consumed = new HashSet<Integer>();
	}
	
	//Functions
	public void clear() {
		pressed.clear();
		consumed.clear();
	}
	
	public void add(Collection<Integer> pressed, Collection<Integer> released) {
		for (Integer i : pressed) {
			if (i != null) setKeyPressed(i);
		}
		for (Integer i : released) {
			if (i != null) setKeyReleased(i);
		}
	}
	
	public boolean consumeKey(int key) {
		if (!isEnabled()) return false;

		if (pressed.contains(key)) {
			return consumed.add(key); //True if the key was not in the list
		}
		return false;
	}
	
	//Getters
	public boolean isEnabled() {
		return enabled;
	}
	public boolean isKeyPressed(int key) {
		if (!isEnabled()) return false;
		
		return pressed.contains(key) && !consumed.contains(key);
	}
	public boolean isKeyHeld(int key) {		
		if (!isEnabled()) return false;
		
		return pressed.contains(key);
	}
	public boolean isKeyConsumed(int key) {
		return consumed.contains(key);
	}
	
	//Setters
	public void setEnabled(boolean e) {
		enabled = e;
	}
	public void setKeyPressed(int key) {
		pressed.add(key);
	}
	public void setKeyReleased(int key) {
		pressed.remove(key);
		consumed.remove(key);
	}
	
}
