package mud.client.view;

import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.util.HashSet;
import java.util.Set;

import javax.imageio.ImageIO;

import mud.common.image.ImageCache;

/**
 * Loads images from disk when they are needed. Can also download them from a
 * URL if they're not locally available.
 */
public class ClientImageCache extends ImageCache {
	
	private Thread imageRequestThread;
	private boolean quit;
	
	private String localURL;
	private String serverURL;
	private Set<String> requestedImages;
	
	public ClientImageCache(String lurl, String surl) {
		this.localURL = lurl;
		this.serverURL = surl;
		
		requestedImages = new HashSet<String>();
		imageRequestThread = new Thread(new Runnable() {
			public void run() {
				while (!quit) {
					String array[];
					
					synchronized (imageRequestThread) {
						try {
							imageRequestThread.wait();
						} catch (InterruptedException e) { }

						array = requestedImages.toArray(new String[requestedImages.size()]);
						requestedImages.clear();
					}
					
					for (String path : array) {
						BufferedImage image = null;
						String localPath = localURL + "/" + path;
						
						try {
							//Try to load from disc
							image = ImageIO.read(new File(localPath));
						} catch (IOException ioe) {
							if (serverURL != null) {
								try {
									//Try to load from server
									image = ImageIO.read(new URL(serverURL + "/" + path));
									ImageIO.write(image, "png", new File(localPath));
								} catch (IOException ioe2) {								
								}
							}
						}
						
						if (image != null) {
							setImage(path, image);
						} else {
							System.err.printf("Unable to load image: %s\n", path);
						}
					}
				}
			}
		});
		imageRequestThread.start();
	}
	
	//Functions
	public void dispose() {
		quit = true;

		super.dispose();
	}
	
	//Getters
	public synchronized BufferedImage getImage(String filename) {
		BufferedImage image = super.getImage(filename);
		if (image == null) {
			requestImage(filename);
		}
		return image;
	}
	
	//Setters
	public void requestImage(String filename) {
		synchronized (imageRequestThread) {
			requestedImages.add(filename);
			imageRequestThread.notify();
		}
	}
	
}
