package mud.client.view;

import java.awt.Color;
import java.awt.Graphics2D;

import mud.common.Constants;
import mud.common.image.ImageRect;
import mud.common.objects.Creature;
import mud.common.objects.GameObject;
import mud.common.objects.Monster;
import mud.common.objects.Player;
import mud.common.objects.WalkActivity;
import mud.common.room.GameTile;

/** Draws an object */
public class GameObjectDrawable implements TiledDrawable {

	private static final boolean debug = false;

	//Functions
	public void draw(GameView gameView, GameTile tile, Graphics2D g, int tileSize) {		
		if (debug) {
			g.setColor(Color.RED);
			g.fillOval(0, 0, tileSize, tileSize);
		}

		if (tile != null) {
			for (GameObject go : tile.getObjects()) {
				ImageRect original = go.getImage();
				if (original != null) {
					
					int tx = Constants.TILE_SIZE * (int)((System.currentTimeMillis()/250) & 3L);
					int ty = go.getDirection().asInt() * Constants.TILE_SIZE;									
					ImageRect derived = new ImageRect(original.getFilename(),
							original.getX()+tx, original.getY()+ty, original.getW(), original.getH());

					int dx = 0;
					int dy = 0;
					if (go.getActivity() instanceof WalkActivity) {
						WalkActivity wa = (WalkActivity)go.getActivity();
						dx += wa.getDx();
						dy += wa.getDy();
					}
					gameView.drawImageRect(g, derived, dx, dy, tileSize, tileSize);
					
					if (go instanceof Player || go instanceof Monster) {
						Creature goc = (Creature) go;
						gameView.drawIdentication(g, goc.getName() + " " + goc.getHp(), dx, dy, tileSize);
					}
				}
			}
		}
	}
	
	//Getters
	
	//Setters
	
}
