package mud.client.view;

import java.awt.AWTException;
import java.awt.BufferCapabilities;
import java.awt.Canvas;
import java.awt.Dimension;
import java.awt.Graphics2D;
import java.awt.ImageCapabilities;
import java.awt.BufferCapabilities.FlipContents;
import java.awt.Color;
import java.awt.RenderingHints;
import java.awt.image.BufferStrategy;
import java.awt.image.BufferedImage;
import java.lang.reflect.InvocationTargetException;

import javax.swing.SwingUtilities;

import mud.common.image.ImageRect;
import mud.common.room.Room;

/** Draws the room etc. */

@SuppressWarnings("serial")
public class GameView extends Canvas {

	private ClientImageCache imageCache;
	private RoomView roomView;
	
	public GameView() {
		setPreferredSize(new Dimension(640, 480));
	}
	
	//Functions
	public void start() {		
		//Create helper objects
		imageCache = new ClientImageCache("img", null);
		roomView = new RoomView();
	}
	public void stop() {
		//Dispose helper objects
		imageCache.dispose();
		imageCache = null;
		roomView = null;
		
		//Dispose buffer
		BufferStrategy bs = getBufferStrategy();
		if (bs != null) bs.dispose();
	}
	
	public void draw(Room room) {
		final BufferStrategy bs = getBufferStrategy();
		if (bs == null || bs.contentsLost()) {
			try {
				createBufferStrategy(2, new BufferCapabilities(new ImageCapabilities(true),
						new ImageCapabilities(true), FlipContents.UNDEFINED));
			} catch (AWTException e) {
				//The above method may fail when not in full-screen mode
				//This is our backup plan
				createBufferStrategy(2);
			}
			return;
		}
		
		Graphics2D g = (Graphics2D)bs.getDrawGraphics();
		roomView.draw(this, room, g, getWidth(), getHeight());
		
		try {
			SwingUtilities.invokeAndWait(new Runnable() {
				public void run() {
					bs.show();
				}
			});
		} catch (InterruptedException e) {
			e.printStackTrace();
		} catch (InvocationTargetException e) {
			e.printStackTrace();
		}
	}
	
	public void drawImageRect(Graphics2D g, ImageRect imageRect, int x, int y, int w, int h) {
		if (imageRect != null) {
			BufferedImage image = imageRect.getImage(imageCache);
			if (image != null) {
				int ix = imageRect.getX();
				int iy = imageRect.getY();
				int iw = imageRect.getW();
				int ih = imageRect.getH();
				g.drawImage(image, x, y, x+w, y+h, ix, iy, ix+iw, iy+ih, this);
			}
		}
	}

	public void drawIdentication(Graphics2D g, String name, int dx, int dy, int tileSize) {
		int fw = g.getFontMetrics().stringWidth(name);
		int fx = (tileSize / 2) - (fw / 2);
		dx += fx;
		dy -= 10;
		g.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
		g.setColor(new Color(0, 0, 0, 100));
		g.fillRoundRect(dx - 5, dy - 8, fw + 10, 16, 18, 18);
		g.setColor(Color.WHITE);
		g.drawString(name, dx, dy + 4);
	}
	
	//Getters
	
	//Setters

}
