package mud.client.view;

import static mud.common.Constants.TILE_SIZE;

import java.awt.Color;
import java.awt.Graphics2D;

import mud.common.room.Room;

/** Draws the room */
public class RoomView {
	
	private GameTileDrawable tileDrawable;
	private GameObjectDrawable objectDrawable;
	
	public RoomView() {
		tileDrawable = new GameTileDrawable();
		objectDrawable = new GameObjectDrawable();
	}
	
	//Functions
	public void draw(GameView gameView, Room room, Graphics2D g, int w, int h) {
		g.setBackground(Color.BLACK);
		g.clearRect(0, 0, w, h);
		
		int dx = 0;
		int dy = 0;
		if (room != null) {			
			dx = (w - room.getWidth()*TILE_SIZE)>>1;
			dy = (h - room.getHeight()*TILE_SIZE)>>1;
		}
			
		g.translate(dx, dy);		
		drawTiled(gameView, room, tileDrawable, g, w, h);
		drawTiled(gameView, room, objectDrawable, g, w, h);		
		g.translate(-dx, -dy);
	}
	
	protected void drawTiled(GameView gameView, Room room, TiledDrawable td, Graphics2D g, int w, int h) {
		int tilesW = w / TILE_SIZE;
		int tilesH = h / TILE_SIZE;

		for (int y = 0; y < tilesH; y++) {
			for (int x = 0; x < tilesW; x++) {
				td.draw(gameView, (room != null ? room.getTile(x, y) : null), g, TILE_SIZE);				
				g.translate(TILE_SIZE, 0);
			}
			g.translate(-tilesW * TILE_SIZE, TILE_SIZE);
		}
		g.translate(0, -tilesH * TILE_SIZE);
	}
	
	//Getters
	
	//Setters
	
}
