package mud.client.view.contextmenu;

import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;

import mud.client.LocalPlayer;
import mud.common.Utils;
import mud.common.items.Item;
import mud.common.items.UsableItem;
import mud.common.objects.Player;
import mud.common.room.GameTile;
import mud.common.room.TileAction;

/** The contextmenu. Activated when the user presses X */

@SuppressWarnings("serial")
public class ContextMenu extends JPopupMenu {
	
	private JMenu itemMenu;
	private JMenu actionMenu;
	
	public ContextMenu() {
		JPopupMenu.setDefaultLightWeightPopupEnabled(false);
		
		add(itemMenu = createItemMenu());
		add(actionMenu = createActionMenu());
		
		add(new ExitItem(this));
	}
	
	//Functions
	public void show(final LocalPlayer lp, Component invoker, int x, int y) {
		Player player = lp.getPlayer();
		
		itemMenu.removeAll();
		if (player.getInventory().getItems().isEmpty()) {
			JMenuItem mi = new JMenuItem("<html><font color='#888888'>&lt;no items in inventory&gt;</font></html>");
			mi.setEnabled(false);
			itemMenu.add(mi);
		} else {		
			for (Item i : player.getInventory().getItems()) {
				String prefix = "";
				if (i == player.getWeapon() || i == player.getArmor()) {
					prefix = "(E) ";
				}
				JMenuItem mi = new JMenuItem(prefix + i.getLabel());
				mi.setEnabled(false);
				itemMenu.add(mi);
	
				if (i instanceof UsableItem) {
					mi.setEnabled(true);
					final UsableItem ui = (UsableItem)i;
					if (ui.canUse()) {
						mi.addActionListener(new ActionListener() {
							public void actionPerformed(ActionEvent e) {
								lp.addContextMenuEvent(new ContextItemEvent(ui));
							}
						});
					}
				}
			}
		}
		
		actionMenu.removeAll();
		GameTile nextTile = player.getRoom().getTile(
				player.getX()+Utils.getDeltaX(player.getDirection(), 1),
				player.getY()+Utils.getDeltaY(player.getDirection(), 1));
		if (nextTile == null || nextTile.getActions().isEmpty()) {
			JMenuItem mi = new JMenuItem("<html><font color='#888888'>&lt;no actions possible&gt;</font></html>");
			mi.setEnabled(false);
			actionMenu.add(mi);
		} else {		
			for (final TileAction action : nextTile.getActions()) {
				JMenuItem mi = new JMenuItem(action.getLabel());
				mi.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						lp.addContextMenuEvent(new ContextActionEvent(action));
					}
				});		
				actionMenu.add(mi);
			}
		}
			
		super.show(invoker, -10, 0);		
	}
	
	protected JMenu createItemMenu() {
		JMenu menu = new JMenu("Inventory");
		return menu;
	}
	
	protected JMenu createActionMenu() {
		JMenu menu = new JMenu("Action");		
		return menu;
	}
	
	//Getters
	
	//Setters
	
}
