package mud.common;

import java.util.Arrays;
import java.util.Random;

public class StringUtil {

	public static String filterId(String id) {
		return filterID(0, id);
	}
	public static String filterID(int offset, String s) {
		if (s == null) {
			return "";
		}
		
		StringBuilder newString = new StringBuilder();
		for (int n = 0; n < s.length(); n++) {
			char c = s.charAt(n);
			
			if ((c >= 'a' && c <= 'z') || (c >= 'A' && c <= 'Z')) {
				newString.append(c);
			} else if (offset > 0 || newString.length() > 0) {
				if ((c >= '0' && c <= '9') || (c == '_')) {
					newString.append(c);
				}
			}
		}		
		return newString.toString();
	}
	public static boolean isValidId(String s) {
		if (s == null || s.length() == 0) return false;
		
		int c[] = toUnicodeArray(s);
		for (int n = 0; n < c.length; n++) {
			if ((c[n] >= 'a' && c[n] <= 'z') || (c[n] >= 'A' && c[n] <= 'Z')) {
				//ok
			} else if (n > 0) {
				if ((c[n] >= '0' && c[n] <= '9') || (c[n] == '_')) {
					//ok
				} else {
					return false;
				}
			} else {
				return false;
			}
		}
		return true;
	}

	public static String replaceAllNoRegex(String string, String pattern, String replacement) {
		StringBuilder buffer = new StringBuilder();		
		
		for (int n = 0; n < string.length(); n++) {
			if (string.regionMatches(n, pattern, 0, pattern.length())) {
				buffer.append(replacement);
				n += pattern.length() - 1;
			} else {
				buffer.append(string.charAt(n));
			}			
		}
		
		return buffer.toString();
	}
		
	public static String stripExtension(String filename) {
		int index = filename.lastIndexOf('.');
		if (index >= 0) {
			return filename.substring(0, index);
		}
		return filename;
	}
	
	public static String toString(int number, int digits) {
		String s = Integer.toString(number);

		StringBuilder sb = new StringBuilder();
		for (int n = digits - s.length(); n > 0; n--) {
			sb.append('0');
		}
		sb.append(s);
		return sb.toString();
	}
	
	public static String repeatString(String pattern, int times) {
		StringBuilder sb = new StringBuilder();
		for (int n = 0; n < times; n++) {
			sb.append(pattern);
		}
		return sb.toString();
	}
	
	public static String getRandomId() {
		return getRandomString(8);
	}
	public static String getRandomString(int length) {
		return getRandomString(length, new Random());
	}
	public static String getRandomString(int length, Random random) {
		int i = (int)(random.nextFloat()*52.0f);
		char c = (char)(i >= 26 ? 'A'+i-26 : 'a'+i);
		
		if (length > 0) return c+getRandomString(length-1, random);
		return Character.toString(c);
	}
	
	public static int[] toUnicodeArray(String s) {
		int string[] = new int[s.length()];
		char chars[] = s.toCharArray();
		
		int t = 0;
		for (int n = 0; n < chars.length; n++) {
			if (Character.isHighSurrogate(chars[n])) {
				string[t] = s.codePointAt(n);
				n++;
				t++;
			} else {
				string[t] = chars[n];
				t++;
			}
		}
		
		return Arrays.copyOfRange(string, 0, t);
	}
}
