package mud.common.events;

import static mud.common.Constants.SERVER_EVENT_ADD_OBJECT;
import static mud.common.Constants.SERVER_EVENT_ATTACK;
import static mud.common.Constants.SERVER_EVENT_DROP;
import static mud.common.Constants.SERVER_EVENT_MOVE;
import static mud.common.Constants.SERVER_EVENT_PICKUP;
import static mud.common.Constants.SERVER_EVENT_REMOVE_OBJECT;
import static mud.common.Constants.SERVER_EVENT_ROOM_INIT;
import static mud.common.Constants.SERVER_EVENT_USE;
import static mud.common.Constants.SERVER_EVENT_USE_ACTION;

import java.io.IOException;

import mud.common.Direction;
import mud.common.Utils;
import mud.common.idl.clients.Event;
import mud.common.idl.clients.EventPackage.AddObjectEvent;
import mud.common.idl.clients.EventPackage.AttackEvent;
import mud.common.idl.clients.EventPackage.DropEvent;
import mud.common.idl.clients.EventPackage.MoveEvent;
import mud.common.idl.clients.EventPackage.PickupEvent;
import mud.common.idl.clients.EventPackage.RemoveObjectEvent;
import mud.common.idl.clients.EventPackage.UseActionEvent;
import mud.common.idl.clients.EventPackage.UseEvent;
import mud.common.items.Item;
import mud.common.items.UsableItem;
import mud.common.loader.RoomLoader;
import mud.common.objects.Creature;
import mud.common.objects.GameObject;
import mud.common.objects.WalkActivity;
import mud.common.room.Room;

/**
 * The logic of how to process a server event can be shared between our Java
 * clients. Other clients have to write their own version.
 */
public class ServerEventHandler {
	
	private static RoomLoader loader = new RoomLoader();
	
	protected ServerEventHandler() {
	}

	//Functions
	public static Room process(Room room, Event event) throws IOException {
		GameObject object;
		Creature creature;
		Item item;
		UsableItem ui;
		
		switch (event.discriminator()) {
		case SERVER_EVENT_ROOM_INIT:
			room = loader.loadRoom(event.roomInitEvent().room_xml);
			break;
		case SERVER_EVENT_MOVE:
			MoveEvent me = event.moveEvent();
			object = room.getObjectById(me.object_id);
			int oldX = object.getX();
			int oldY = object.getY();
			object.setPos(me.x, me.y);
			if (Direction.fromInt(me.direction) != null) {
				//Only set the direction if it's valid
				object.setDirection(Direction.fromInt(me.direction));
				
				if (oldX != object.getX() || oldY != object.getY()) {
					//Only start a move activity if actually moved
					object.setActivity(new WalkActivity(Utils.getDeltaX(object.getDirection(), 1),
						Utils.getDeltaY(object.getDirection(), 1)));
				}
			}
			break;
		case SERVER_EVENT_USE:
			UseEvent ue = event.useEvent();
			creature = (Creature)room.getObjectById(ue.creature_id);
			ui = (UsableItem)creature.getInventory().getItemById(ue.item_id);
			creature.getInventory().removeItem(ui);
			ui.use();
			break;
		case SERVER_EVENT_DROP:
			DropEvent de = event.dropEvent();
			creature = (Creature)room.getObjectById(de.creature_id);
			ui = (UsableItem)creature.getInventory().getItemById(de.item_id);
			creature.getInventory().removeItem(ui);
			break;
		case SERVER_EVENT_REMOVE_OBJECT:
			RemoveObjectEvent roe = event.removeObjectEvent();
			room.removeObject(room.getObjectById(roe.object_id));
			break;
		case SERVER_EVENT_ADD_OBJECT:
			AddObjectEvent aoe = event.addObjectEvent();
			room.addObject(loader.loadObject(aoe.object_xml));
			break;
		case SERVER_EVENT_ATTACK:
			{
				AttackEvent ae = event.attackEvent();
				Creature attacker = (Creature)room.getObjectById(ae.attacker_id);
				Creature defender = (Creature)room.getObjectById(ae.defender_id);
				attacker.setHp(attacker.getHp() - ae.attacker_damage);
				defender.setHp(defender.getHp() - ae.defender_damage);
			}
			break;
		case SERVER_EVENT_PICKUP:
			PickupEvent pe = event.pickupEvent();
			item = loader.loadItem(pe.item_xml);
			creature = (Creature)room.getObjectById(pe.creature_id);
			creature.getInventory().addItem(item);
			break;
		case SERVER_EVENT_USE_ACTION:
			{
				UseActionEvent uae = event.useActionEvent();
				Creature user = (Creature)room.getObjectById(uae.user_id);
				GameObject target = room.getObjectById(uae.target_id);
				target.executeTileAction(user, uae.action_id);
			}
			break;
		default:
			System.err.printf("Received event of unknown type: %d\n", event.discriminator());
			break;
		}
		
		return room;
	}
	
	//Getters
	
	//Setters
	
}
