package mud.common.image;

import java.awt.Rectangle;
import java.awt.image.BufferedImage;

/** A serializable way of specifying a sub-rectangle of an image */
public class ImageRect {
	
	private String filename;
	private Rectangle rect;
	
	public ImageRect(String filename, int x, int y, int w, int h) {
		this(filename, new Rectangle(x, y, w, h));
	}
	public ImageRect(String filename, Rectangle rect) {
		this.filename = filename;
		this.rect = rect;
	}
	
	//Functions
	public static ImageRect fromString(String s) {
		String parts[] = s.split(":");
		if (parts.length != 5) {
			throw new IllegalArgumentException("String has wrong format: " + s);
		}
		
		try {
			return new ImageRect(parts[0], Integer.parseInt(parts[1]), Integer.parseInt(parts[2]),
					Integer.parseInt(parts[3]), Integer.parseInt(parts[4]));
		} catch (NumberFormatException nfe) {
			throw new IllegalArgumentException("String has wrong format: " + s);			
		}
	}
	public String toString() {
		return String.format("%s:%d:%d:%d:%d", filename, rect.x, rect.y, rect.width, rect.height);
	}
	
	//Getters
	public BufferedImage getImage(ImageCache cache) { return cache.getImage(filename); }
	public String getFilename() { return filename; }
	public int getX() { return rect.x; }
	public int getY() { return rect.y; }
	public int getW() { return rect.width; }
	public int getH() { return rect.height; }
	
	//Setters
	
}
