package mud.common.objects;

import mud.common.image.ImageRect;
import mud.common.xml.XmlElement;

/** Living object */
public abstract class Creature extends GameObject {

	public enum Direction {
		NORTH(0), SOUTH(1), EAST(2), WEST(3);
		
		private int intval;
		
		private Direction(int intval) {
			this.intval = intval;
		}
		
		public int asInt() {
			return intval;
		}
		public static Direction fromInt(int intval) {
			switch (intval) {
			case 0: return Direction.NORTH;
			case 1: return Direction.SOUTH;
			case 2: return Direction.EAST;
			case 3: return Direction.WEST;
			}
			return null;
		}
	}
	
	private int hp;
	private Inventory inventory;
	
	public Creature(int id, ImageRect image, int hp) {
		super(id, image);
		this.hp = hp;
		
		inventory = new Inventory();
	}
	
	//Functions	
	public void destroy() {
		getRoom().removeObject(this);		
	}	
	protected void onDestroyed() {
		destroy();
	}
	
	//Getters
	public int getHp() { return hp; }
	public Inventory getInventory() { return inventory; }
	public abstract int getAttack();
	public abstract int getDefense();

	//Setters
	public void setHp(int h) {
		hp = h;
		if (hp <= 0) {
			onDestroyed();
		}
	}
		
	//Save Support
	protected abstract String getXmlTagName();
	protected void toXml2(XmlElement e) {
		super.toXml2(e);
		
		e.addChild("hp", getHp());
		e.addChild(inventory.toXml());
	}
}
