package mud.common.objects;

import java.util.Collection;

import mud.common.image.ImageRect;
import mud.common.Direction;
import mud.common.room.GameTile;
import mud.common.room.IdObject;
import mud.common.room.Room;
import mud.common.room.TileAction;
import mud.common.xml.XmlElement;

/** Anything that can stand on top of a tile */
public class GameObject extends IdObject {

	private Room room;
	private ImageRect image;
    private String name;
	private Direction direction;
	private int x;
	private int y;

	private Activity activity;
	
	public GameObject(int id, ImageRect image) {
		super(id);
		
		this.image = image;
		this.name = "unnamed";
		this.direction = Direction.SOUTH;
	}
	
	//Functions
	public void dispose() {
		room.removeObject(this);
	}
	public void contributeTileActions(Collection<TileAction> c) {
	}
	public void executeTileAction(Creature user, int actionId) {
	}
	
	//Getters
	public ImageRect getImage() { return image; }
	public Room getRoom() { return room; }
	public GameTile getTile() { return (room != null ? room.getTile(x, y) : null); }
	public Activity getActivity() {
		if (activity != null && activity.isFinished()) {
			activity = null;
		}
		return activity;
	}
	public Direction getDirection() { return direction; }
	public int getX() { return x; }
	public int getY() { return y; }
	public String getName(){ return name; }
	
	//Setters
	public void setRoom(Room r) {
		if (room != null) room.getTile(x, y).removeOccupant(this);
		
		room = r;
		
		if (room != null) room.getTile(x, y).addOccupant(this);
	}
	public void setDirection(Direction d) {
		if (d == null) throw new IllegalArgumentException("null values not allowed");
		
		direction = d;
	}
	public void setPos(int targetX, int targetY) {
		if (x == targetX && y == targetY) {
			return;
		}
		
		if (room != null) room.getTile(x, y).removeOccupant(this);

		x = targetX;
		y = targetY;

		if (room != null) room.getTile(x, y).addOccupant(this);
	}
	public void setActivity(Activity a) {
		this.activity = a;
	}
	protected void setName(String n) { name = n; }
		
	//Save Support
	protected String getXmlTagName() {
		return "gameObject";
	}
	public final XmlElement toXml() {
		XmlElement objectE = new XmlElement(getXmlTagName());
		toXml2(objectE);
		return objectE;
	}
	protected void toXml2(XmlElement e) {
		e.addAttribute("id", getId());
		
		e.addChild("image", image.toString());
		e.addChild("direction", direction.asInt());
		e.addChild("x", x);
		e.addChild("y", y);
		e.addChild("name",name);
	}
	
}
