package mud.common.objects;

import java.util.Collections;
import java.util.LinkedList;
import java.util.List;

import mud.common.items.Item;
import mud.common.xml.XmlElement;

public class Inventory {

	private List<Item> items;
	private List<InventoryListener> listeners;
	
	public Inventory() {
		items = new LinkedList<Item>();
		listeners = new LinkedList<InventoryListener>();
	}
	
	//Functions
	public void addListener(InventoryListener il) {
		listeners.add(il);
	}
	public void removeListener(InventoryListener il) {
		listeners.remove(il);
	}
	
	public boolean isEmpty()
	{
		return items.isEmpty();
	}
	
	public void clear() {
		items.clear();
	}
	public void addAll(Inventory inventory) {
		for (Item i : inventory.getItems()) {
			addItem(i);
		}
	}
	public void addItem(Item i) {
		items.add(i);
		
		//Fire an event, copy the listeners list so they can add/remove themselves during the event handling 
		InventoryListener ils[] = listeners.toArray(new InventoryListener[listeners.size()]);
		for (InventoryListener il : ils) il.onItemAdded(i);
	}
	public Item removeItem(int itemId) {
		for (Item i : items) {
			if (i.getId() == itemId) {
				return removeItem(i);
			}
		}
		return null;
	}
	public Item removeItem(Item i) {
		items.remove(i);

		//Fire an event, copy the listeners list so they can add/remove themselves during the event handling 
		InventoryListener ils[] = listeners.toArray(new InventoryListener[listeners.size()]);
		for (InventoryListener il : ils) il.onItemRemoved(i);
		
		return i;
	}
	
	//Getters
	public List<Item> getItems() {
		return Collections.unmodifiableList(items);
	}
	public Item getItemById(int itemId) {
		for (Item i : items) {
			if (i.getId() == itemId) {
				return i;
			}
		}
		return null;
	}
	
	//Setters
	
	
	//Save Support
	public XmlElement toXml() {
		XmlElement inventoryE = new XmlElement("inventory");
		
		XmlElement itemsE = inventoryE.addChild("items");
		for (Item i : items) {
			itemsE.addChild(i.toXml());
		}
		
		return inventoryE;
	}
}
