package mud.common.objects;

import mud.common.image.ImageRect;
import mud.common.items.Armor;
import mud.common.items.Item;
import mud.common.items.Weapon;
import mud.common.xml.XmlElement;

public class Player extends Creature {

	private InventoryListener inventoryListener;
	private Weapon weapon;
	private Armor armor;
	
	public Player(int id, String name, ImageRect image, int hp) {
		super(id, image, hp);

		setName(name);
		getInventory().addListener(inventoryListener = new PlayerInventoryListener());
	}

	//Functions
	public void dispose() {
		getInventory().removeListener(inventoryListener);
		
		super.dispose();
	}	
	
	/** Sets the active weapon to the strongest one in the player's inventory */
	protected void updateWeapon() {
		weapon = null;
		for (Item i : getInventory().getItems()) {
			if (i instanceof Weapon) {
				Weapon w = (Weapon)i;
				if (weapon == null || w.getAttack() > weapon.getAttack()) {
					weapon = w;
				}
			}
		}
	}
	
	/** Sets the active armor to the strongest one in the player's inventory */
	protected void updateArmor() {
		armor = null;
		for (Item i : getInventory().getItems()) {
			if (i instanceof Armor) {
				Armor a = (Armor)i;
				if (armor == null || a.getDefense() > armor.getDefense()) {
					armor = a;
				}
			}
		}
	}
	
	//Getters
	public int getAttack() {
		return (weapon != null ? weapon.getAttack() : 0);
	}
	public int getDefense() {
		return (armor != null ? armor.getDefense() : 0);
	}
	public Weapon getWeapon() { return weapon; }
	public Armor getArmor() { return armor; }
	
	//Setters

	//Save Support
	protected String getXmlTagName() { return "player"; }
	protected void toXml2(XmlElement e) {
		super.toXml2(e);
				
	}
	
	//Inner Classes
	private class PlayerInventoryListener implements InventoryListener {

		public void onItemAdded(Item i) {
			updateWeapon();
			updateArmor();
		}
		public void onItemRemoved(Item i) {
			if (i == weapon) {
				updateWeapon();
			} else if (i == armor) {
				updateArmor();
			}
		}
		
	}
	
}
