package mud.common.room;

import java.util.ArrayList;
import java.util.Collections;
import java.util.LinkedList;
import java.util.List;

import mud.common.image.ImageRect;
import mud.common.objects.GameObject;
import mud.common.xml.XmlElement;

/**
 * Implements a single tile.
 * @author Pieter van Ede
 *
 */
public class GameTile {
	
	private ImageRect image;
	private List<GameObject> occupants;
	private boolean solid;

	public GameTile(ImageRect image) {
		this.image = image;
		
		occupants = new LinkedList<GameObject>();
	}

	//Functions
	public void addOccupant(GameObject object) {
		if (!occupants.contains(object)) {
			occupants.add(object);
		}
	}
	public void removeOccupant(GameObject object) {
		occupants.remove(object);
	}
	public void removeOccupant(int index) {
		if (index < 0 || index >= occupants.size()) throw new IllegalArgumentException("GameTile.removeOccupant() :: index is out of range: " + index);
		
		occupants.remove(index);
	}
	
	//Getters
	public ImageRect getImage() {
		return image;
	}
	public List<TileAction> getActions() {
		List<TileAction> actions = new ArrayList<TileAction>();
		for (GameObject go : occupants) {
			go.contributeTileActions(actions);
		}
		return actions;
	}
	public List<GameObject> getObjects() {
		return Collections.unmodifiableList(occupants);
	}
	public boolean isSolid() {
		return solid;
	}
	public boolean isFree() {
		return !solid && occupants.isEmpty();
	}
	
	//Setters
	public void setSolid(boolean s) {
		this.solid = s;
	}
	
	//Save Support
	public XmlElement toXml() {
		XmlElement tileE = new XmlElement("tile");
		tileE.addChild("image", image.toString());
		tileE.addChild("solid", solid?"true":"false");
		
		XmlElement occupantsE = tileE.addChild("occupants");
		for (GameObject go : occupants) {
			occupantsE.addChild(go.toXml());
		}
		
		return tileE;
	}
	
}
