package mud.common.xml;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

public class XmlReader {

	public XmlReader() {
	}
	
	//Functions
	protected DocumentBuilder createDocumentBuilder() {
		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
        dbf.setNamespaceAware(true);
		
		try {
			return dbf.newDocumentBuilder();
		} catch (ParserConfigurationException e) {
			e.printStackTrace();
		}
		return null;
	}
	
	public XmlElement read(File file) throws SAXException, IOException {
		DocumentBuilder db = createDocumentBuilder();		
		return (XmlElement)convert(db.parse(file));
	}
	public XmlElement read(InputStream in) throws SAXException, IOException {
		DocumentBuilder db = createDocumentBuilder();		
		return (XmlElement)convert(db.parse(in));
	}
	
	public static XmlNode convert(Node node) {
		if (node instanceof Element || node instanceof Document) {
			XmlElement object;

			String prefix = node.getPrefix();
			if (prefix == null) {
				object = new XmlElement(node.getNodeName());
			} else {
				// Resolve the namespace prefix so we don't have to worry about
				// name conflicts

				String name = node.getNodeName();
				name = name.substring(name.indexOf(':') + 1);
				object = new XmlElement(prefix + ":" + name);
			}
			parseAttributes(object, node);

			NodeList list = null;
			if (node instanceof Element) {
				list = ((Element) node).getChildNodes();
			} else if (node instanceof Document) {
				list = ((Document) node).getChildNodes();
			}

			for (int n = 0; n < list.getLength(); n++) {
				object.addChild(convert(list.item(n)));
			}

			return object;
		} else {
			return new XmlTextNode(node.getNodeValue());
		}
	}
	private static XmlElement parseAttributes(XmlElement object, Node node) {
		NamedNodeMap attributes = node.getAttributes();
		if (attributes != null) {
			for (int n = 0; n < attributes.getLength(); n++) {
				Node a = attributes.item(n);
				object.addAttribute(a.getNodeName(), a.getNodeValue());
			}
		}
		return object;
	}

	//Getters

	//Setters

}
