package mud.common.xml;

import mud.common.StringUtil;

public class XmlUtil {

	private static String level0Escape(String string) {
		string = StringUtil.replaceAllNoRegex(string, "&", "&amp;");
		string = StringUtil.replaceAllNoRegex(string, "\"", "&quot;");
		string = StringUtil.replaceAllNoRegex(string, "<", "&lt;");
		string = StringUtil.replaceAllNoRegex(string, ">", "&gt;");
		
		return string;
	}
	private static String level0Unescape(String string) {
		string = StringUtil.replaceAllNoRegex(string, "&gt;", ">");
		string = StringUtil.replaceAllNoRegex(string, "&lt;", "<");
		string = StringUtil.replaceAllNoRegex(string, "&quot;", "\"");
		string = StringUtil.replaceAllNoRegex(string, "&amp;", "&");
		
		return string;
	}

	private static String level1Escape(String string) {
		string = level0Escape(string);
		
		StringBuilder sb = new StringBuilder();
		for (int n = 0; n < string.length(); n++) {
			char c = string.charAt(n);
			boolean ok = (Character.isLetter(c) || c == '_' || c == ':');
			if (!ok && n > 0) {
				ok = (Character.isDigit(c) || c == '.' || c == '-');
			}
			
			if (!ok) {
				sb.append('_');
			} else {
				sb.append(c);
			}
		}
		
		return sb.toString();
	}
	
	public static String tagEscape(String tag) {
		tag = level0Escape(tag);
		
		return escapeToUTF8(tag);
	}
	public static String tagUnescape(String tag) {
		return level0Unescape(tag);
	}

	public static String attributeNameEscape(String name) {
		name = level1Escape(name);
		
		return escapeToUTF8(name);
	}

	public static String attributeValueEscape(String value) {
		value = level0Escape(value);
				
		return escapeToUTF8(value);
	}
	
	public static String escapeToUTF8(String string) {
		StringBuilder sb = new StringBuilder();
		
		for (int n = 0; n < string.length(); n++) {
			int c = string.codePointAt(n);
			if (c > 127) {
				sb.append("&#");
				sb.append(c);
				sb.append(';');
			} else {
				sb.append((char)c);
			}
		}
		
		return sb.toString();
	}
	
	public static String stripTags(String text) {
		StringBuilder sb = new StringBuilder();
		
		boolean inTag = false;
		for (int n = 0; n < text.length(); n++) {
			char c = text.charAt(n);
			
			if (c == '<') inTag = true;
			
			if (!inTag) {
				sb.append(c);
			}

			if (c == '>') inTag = false;
		}
		
		return sb.toString();
	}
	
	public static String encodeRGB(int data[]) {
		StringBuilder sb = new StringBuilder();
		for (int i : data) {
			String s = Integer.toHexString(i);
			for (int t = s.length(); t < 6; t++) {
				sb.append('0');
			}
			sb.append(s.substring(Math.max(0, s.length() - 6), s.length()));
		}		
		return sb.toString();
	}
	
	public static int[] decodeRGB(String data) {
		int rgb[] = new int[data.length() / 6];
		int x = 0;
		for (int n = 0; n < data.length(); n += 6) {
			int color = 0;
			for (int t = 0; t < 6; t++) {
				int c = data.charAt(t);
				if (c >= '0' && c <= '9') {
					c -= '0';
				} else {
					if (c >= 'a' && c <= 'f') {
						c = 10 + (c - 'a');
					} else if (c >= 'A' && c <= 'F') {
						c = 10 + (c - 'A');
					} else {
						throw new NumberFormatException("Unknown char: " + c);
					}
				}
				color = (color << 4) | c;
			}
			rgb[x] = color;
			x++;
		}
		return rgb;
	}
	
	public static String getString(XmlElement element, String childName) {
		XmlNode child = element.getChild(childName);
		if (child != null) {
			return child.getTextContent();
		}
		return null;
	}
	public static int getInt(XmlElement element, String childName, int defaultValue) {
		String string = getString(element, childName);
		if (string != null) {
			try {
				return Integer.parseInt(string);				
			} catch (NumberFormatException nfe) {}
		}
		return defaultValue;
	}
	public static long getLong(XmlElement element, String childName, long defaultValue) {
		String string = getString(element, childName);
		if (string != null) {
			try {
				return Long.parseLong(string);				
			} catch (NumberFormatException nfe) {}
		}
		return defaultValue;
	}
	public static boolean getBoolean(XmlElement element, String childName, boolean defaultValue) {
		String string = getString(element, childName);
		if (string != null) {
			try {
				return Boolean.parseBoolean(string);				
			} catch (NumberFormatException nfe) {}
		}
		return defaultValue;
	}
	
}
