package mud.server;

import java.util.Collection;
import java.util.Iterator;

import mud.common.room.Room;
import mud.server.corba.GameLogic;
import mud.server.corba.ServerImpl;

/** Manages the AI for the monsters */
public class MonsterController {

	private MonsterThread threads[];
	
	public MonsterController() {		
	}
	
	//Functions
	public void start(ServerImpl server, GameLogic gameLogic, Collection<Room> rooms) {
		if (isRunning()) {
			throw new IllegalStateException("MonsterController.start() :: Already running");
		}
		
		//Spawn a thread for each room, rooms are completely independent so we can do this 
		threads = new MonsterThread[rooms.size()];
		Iterator<Room> i = rooms.iterator();
		for (int n = 0; n < rooms.size(); n++) {
			threads[n] = new MonsterThread(server, gameLogic, i.next());
			threads[n].start();
		}
	}
	public void stop() {
		for (int n = 0; n < threads.length; n++) {
			threads[n].stop();
		}
		threads = null;
	}
	
	//Getters
	protected boolean isRunning() {
		if (threads != null) {
			for (MonsterThread t : threads) {
				if (t.isRunning()) return true;
			}
		}
		return false;
	}
	
	//Setters
}
