package mud.server;

import static mud.server.corba.ServerImpl.random;

import java.util.Collection;

import mud.common.objects.GameObject;
import mud.common.objects.Monster;
import mud.common.objects.Player;
import mud.common.room.Room;
import mud.server.corba.GameLogic;
import mud.server.corba.ServerImpl;

/** Controls all the monsters in a single room */
public class MonsterThread implements Runnable {

	private Thread thread;
	private boolean running;
	private boolean quit;

	private ServerImpl server;
	private GameLogic gameLogic;
	private Room room;
	
	public MonsterThread(ServerImpl server, GameLogic gameLogic, Room room) {
		this.server = server;
		this.gameLogic = gameLogic;
		this.room = room;
	}
	
	//Functions
	public void start() {
		if (running) throw new IllegalArgumentException("MonsterThread.start() :: Already running");
		
		running = true;
		thread = new Thread(this, "MonsterThread::" + room.getId());
		thread.start();
	}
	public void stop() {
		thread = null;
		quit = true;
	}
	
	public void run() {
		while (!quit) {
			GameObject objects[] = room.getObjects().toArray(new GameObject[0]);
			for (GameObject go : objects) {
				if (go instanceof Monster) {
					synchronized (server) {
						updateMonster((Monster)go);
					}
				}
			}
			
			try {
				Thread.sleep(250);
			} catch (InterruptedException e) { }
		}
		running = false;
	}

	protected void updateMonster(Monster monster) {
		int dx = 0;
		int dy = 0;
		
		//Calculate distance to every square
		short dist[][] = new short[room.getWidth()][room.getHeight()];
		for (int x = 0; x < dist.length; x++) {
			for (int y = 0; y < dist[x].length; y++) {
				dist[x][y] = Short.MAX_VALUE;
			}
		}
		floodFillDist(room, dist, monster.getX(), monster.getY(), (short)0);
		
		//Find closest player
		Player player = findClosestPlayer(room.getObjects(), dist);
		if (player == null) {
			int dxs[] = {-1, 1, 0, 0};
			int dys[] = {0, 0, -1, 1};
			
			int r = random.nextInt();
			dx = dxs[r&3];
			dy = dys[r&3];
		} else {
			if (player.getX() < monster.getX()) {
				dx = -1;
			} else if (player.getX() > monster.getX()) {
				dx = 1;
			} else if (player.getY() < monster.getY()) {
				dy = -1;
			} else if (player.getY() > monster.getY()) {
				dy = 1;
			}
		}
		
		gameLogic.move(monster, monster.getX()+dx, monster.getY()+dy);
	}
	
	protected void floodFillDist(Room room, short dist[][], int x, int y, short d) {
		if (x < 0 || y < 0 || x >= room.getWidth() || y >= room.getHeight()) {
			return;
		}
		if (room.getTile(x, y).isSolid()) {
			return;
		}
		if (dist[x][y] <= d) {
			return;
		}
		dist[x][y] = d;
		
		short nd = (short)(d+1);
		floodFillDist(room, dist, x-1, y, nd);
		floodFillDist(room, dist, x+1, y, nd);
		floodFillDist(room, dist, x, y-1, nd);
		floodFillDist(room, dist, x, y+1, nd);
	}
	protected Player findClosestPlayer(Collection<GameObject> objects, short dist[][]) {
		Player best = null;
		short bestDist = Short.MAX_VALUE;
		for (GameObject o : objects) {
			if (o instanceof Player && dist[o.getX()][o.getY()] < bestDist) {
				best = (Player)o;
			}
		}
		return best;
	}
	
	//Getters
	public boolean isRunning() { return running; }
	
	//Setters
}
