package mud.server;

import java.util.LinkedList;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 *An Object Orientated implementation of the WaitQueue. (Locked based)
 * 
 * @author Phyx
 */
public class WaitQueue<E> {

	private LinkedList<E> queue;

	/**
	 * Create a new WaitQueue
	 */
	public WaitQueue() {
		queue = new LinkedList<E>();
	}

	/**
	 * Add a new item to the queue
	 * 
	 * @param item Item to add
	 */
	public void push(E item) {
		synchronized (queue) {
			queue.add(item);
			queue.notify();
		}
	}

	/**
	 * Gets an item out of the queue, or blocks untill one is available
	 * 
	 * @return item to use
	 */
	public E pop() {
		E item;
		synchronized (queue) {

			try {
				while (queue.isEmpty()) {
					queue.wait();
				}
				item = queue.pop();
			} catch (InterruptedException ex) {
				Logger.getLogger(WaitQueue.class.getName()).log(Level.SEVERE, null, ex);
				item = null;
			}
		}

		return item;
	}
}
