package mud.server.corba;

import java.util.Properties;
import mud.common.Constants;
import mud.common.idl.servers.Server;
import mud.common.idl.servers.ServerHelper;
import org.omg.CORBA.ORB;
import org.omg.CORBA.Object;
import org.omg.CosNaming.NameComponent;
import org.omg.CosNaming.NamingContextExt;
import org.omg.CosNaming.NamingContextExtHelper;
import org.omg.PortableServer.POA;
import org.omg.PortableServer.POAHelper;

public class CorbaServer implements Runnable {

	private ORB orb;
	private Object obj;
	private String[] args;
	private Properties properties;
	private Thread thread;
	private boolean running;

	/**
	 * Create a new CorbaServer instance
	 * 
	 * @param servername Hostname of the server to listen on, if null default
	 *        value will be used
	 * @param port Port to listen on, if null default value will be used
	 * @param args Commandline arguments to pass to the orb, can be null
	 */
	public CorbaServer(String servername, String port, String[] args) {
		properties = new Properties();
		if (port != null) {
			properties.put("org.omg.CORBA.ORBInitialPort", port);
		}
		if (servername != null) {
			properties.put("org.omg.CORBA.ORBInitialHost", servername);
		}
		this.args = args;
	}

	/**
	 * Initialize and register all classes with the orb
	 */
	public void start() {
		thread = new Thread(this);
		thread.start();
		running = true;
	}

	/**
	 * Stop the orb and all related services
	 */
	public void stop() {
		if (!running)
			return;
		orb.shutdown(false);
		orb = null;
		obj = null;
		running = false;
	}

	public void run() {
		try {
			orb = ORB.init(args, properties);

			POA rootPoa = POAHelper.narrow(orb.resolve_initial_references("RootPOA"));
			rootPoa.the_POAManager().activate();

			obj = orb.resolve_initial_references("NameService");

			NamingContextExt ncRef = NamingContextExtHelper.narrow(obj);

			ServerImpl server = new ServerImpl(orb, Constants.BroadCastThreads);

			org.omg.CORBA.Object ref = rootPoa.servant_to_reference(server);
			Server href = ServerHelper.narrow(ref);

			String name = "GameServer";
			NameComponent path[] = ncRef.to_name(name);

			ncRef.rebind(path, href);

			orb.run();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
}
