package mud.server.test;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;

import mud.common.Constants;
import mud.common.image.ImageRect;
import mud.common.items.Armor;
import mud.common.loader.RoomLoader;
import mud.common.objects.Monster;
import mud.common.objects.Transporter;
import mud.common.objects.Treasure;
import mud.common.room.GameTile;
import mud.common.room.Room;

public class RoomSaveTest {
	
	private static int objectIdCounter = 1000000;
	
	public RoomSaveTest() {		
	}
	
	//Functions
	public static void main(String args[]) {
		RoomSaveTest roomSaveTest = new RoomSaveTest();
		roomSaveTest.run(new File("roomSaveTest"));
	}
	
	public void run(File folder) {
		if (!folder.exists()) folder.mkdirs();
		
		createRoom1(folder.getAbsolutePath()+"/room1.xml");
		createRoom2(folder.getAbsolutePath()+"/room2.xml");
	}
	
	protected void saveRoom(Room room, String path) {
		System.out.println("Saving xml as: " + path);
		
		try {
			FileOutputStream fout = new FileOutputStream(path);
			fout.write(room.toXml().toXmlString().getBytes());
			fout.flush();
			fout.close();			
		} catch (IOException ioe) {
			ioe.printStackTrace();
		}
	}
	protected Room loadRoom(String path) {
		try {
			FileInputStream fin = new FileInputStream(path);
			byte[] bytes = new byte[(int)new File(path).length()];
			fin.read(bytes);
			fin.close();
			
			return new RoomLoader().loadRoom(new String(bytes));
		} catch (IOException ioe) {
			ioe.printStackTrace();
		}
		return null;
	}
	
	protected void createRoom1(String path) {
		int w = 20;
		int h = 15;
		
		GameTile tiles[][] = createEmptyTileSet(w, h);
		for (int n = 0; n < w; n++) tiles[n][0]   = tile(0, 0, true);
		for (int n = 0; n < w; n++) tiles[n][h-1] = tile(0, 0, true);
		for (int n = 0; n < h; n++) tiles[0][n]   = tile(0, 0, true);
		for (int n = 0; n < h; n++) tiles[w-1][n] = tile(0, 0, true);
		
		Room room = new Room(1, tiles);
		transporter(room, 0, h/2, 2, w-2, h/2);
		transporter(room, w-1, h/2, 2, 1, h/2);
		
		monster(room, 10, 3);
		
		saveRoom(room, path);
		room = loadRoom(path);
		saveRoom(room, path.substring(0, path.lastIndexOf('.'))+"-resaved.xml");
	}
	
	protected void createRoom2(String path) {
		int w = 20;
		int h = 15;
		
		GameTile tiles[][] = createEmptyTileSet(w, h);
		for (int n = 0; n < w; n++) tiles[n][0]   = tile(0, 0, true);
		for (int n = 0; n < w; n++) tiles[n][h-1] = tile(0, 0, true);
		for (int n = 0; n < h; n++) tiles[0][n]   = tile(0, 0, true);
		for (int n = 0; n < h; n++) tiles[w-1][n] = tile(0, 0, true);
		
		Room room = new Room(2, tiles);
		transporter(room, 0, h/2, 1, w-2, h/2);
		transporter(room, w-1, h/2, 1, 1, h/2);
		
		Monster monster = monster(room, 15, 9);
		monster.getInventory().addItem(new Armor(nextObjectId(), "Slimy Armor", 20));
		treasure(room, 1, 1);
		
		saveRoom(room, path);
		room = loadRoom(path);
		saveRoom(room, path.substring(0, path.lastIndexOf('.'))+"-resaved.xml");
	}
	
	protected Monster monster(Room room, int x, int y) {
		Monster monster = new Monster(nextObjectId(), image("sprite.png", 4, 4), 20, 2, 0);
		room.addObject(monster);
		monster.setPos(x, y);
		return monster;
	}
	protected Transporter transporter(Room from, int x, int y, int target, int tx, int ty) {
		Transporter transporter = new Transporter(nextObjectId(), image("tileset.png", 2, 0),
				target, tx, ty);
		from.addObject(transporter);
		transporter.setPos(x, y);
		return transporter;
	}
	protected Treasure treasure(Room room, int x, int y) {
		Treasure treasure = new Treasure(nextObjectId(), image("sprite.png", 4, 0));
		room.addObject(treasure);
		treasure.getInventory().addItem(new Armor(nextObjectId(), "Dusty Armor", 3));
		treasure.setPos(x, y);
		return treasure;
	}
	
	protected GameTile[][] createEmptyTileSet(int w, int h) {
		GameTile tiles[][] = new GameTile[20][15];
		for (int x = 0; x < tiles.length; x++) {
			for (int y = 0; y < tiles[x].length; y++) {
				tiles[x][y] = tile(1, 0, false);
			}
		}
		return tiles;
	}
	
	//Getters
	protected int nextObjectId() {
		objectIdCounter++;
		return objectIdCounter;
	}
	protected ImageRect image(String image, int x, int y) {
		return new ImageRect(image,
				Constants.TILE_SIZE*x, Constants.TILE_SIZE*y,
				Constants.TILE_SIZE, Constants.TILE_SIZE);		
	}
	protected GameTile tile(int x, int y, boolean solid) {
		GameTile tile = new GameTile(image("tileset.png", x, y));
		tile.setSolid(solid);
		return tile;
	}
	
	//Setters
	
}
